require 'uu/os/gvc/component'
require 'uu/os/gvc/table_row'

module UU
  module OS
    module GVC
      
      # Table component.
      class Table < UU::OS::GVC::Component
        
        # Creates new instance of Table.
        #
        # @param [UU::OS::GVC::Container] container Container where to insert new Table
        # @param [String, Hash] data Initialization data in form of JSON string or Hash (optional)
        def initialize(container, data = nil)
          super(container, data)

          if rows.kind_of?Array
            rows.map!{ |row_data|
              UU::OS::GVC::TableRow.new(row_data)
            }
          end
        end

        # Flag if table is visible in form. Default is true. If is set to false table is completely hidden.
        #
        # @example
        #   # get table component
        #   table = view.get_components_by_code("table")[0]
        #
        #   # hide table
        #   table.visible = false
        #
        #   # make table visible again
        #   table.visible = true
        #
        # @return [Boolean]
        attr_accessor :visible

        #  Returns row object on specific index.
        #
        # @example
        #   # get table component
        #   table = view.get_components_by_code("table")[0]
        #
        #   # get third row
        #   row = table.get_row(2) # UU::OS::GVC::TableRow object
        #   code = row.code # string "row_3"
        #
        #   # get last row from five rows
        #   row = table.get_row(-1) # UU::OS::GVC::TableRow object
        #   row.visible = false  # set row visibility
        #
        # @param [Integer] index The Index of row. An index can be represented by a single positive integer between 0 (index 0 = first row)
        #   and the number of rows (excluding) which is returned by {UU::OS::GVC::Table#number_of_rows} method.
        #   You can use negative indices as well, which start counting from the end (i.e. minus number of rows), with -1 being the last element.
        # @return [UU::OS::GVC::TableRow, nil] The table row object for specific index or nil if index is greater than or equal to number_of_rows.
        def get_row(index)
          rows[index]
        end

        # Returns array of table row objects with specific code. If nil is entered as the code, than all rows
        # without a defined code are returned.
        #
        # @example
        #   # get table component
        #   table = view.get_components_by_code("table")[0]
        #
        #   # get row with code "row_4"
        #   row_array = table.get_rows_by_code("row_4") # Array of UU::OS::GVC::TableRow object
        #
        #   # get first item of array (our row object with code "row_4")
        #   row = row_array[0]
        #   code = row.code # string "row_4"
        #   row.visible = false  # set row visibility
        #
        # @param [String] code Table row code.
        # @return [Array] Array of {UU::OS::GVC::TableRow} objects. The array may be empty if there is no row with specific code.
        def get_rows_by_code(code)
          rows.select{|row|
            row.code == code
          }
        end

        # Returns number of rows in Table. May be zero.
        #
        # @example
        #   # get table component
        #   table = view.get_components_by_code("table")[0]
        #
        #   # get number of rows in Table
        #   rows_num = table.number_of_rows # Number 5
        #
        # @return [Numeric] Number of rows.
        def number_of_rows
          rows.size
        end

        alias_method :size, :number_of_rows
       

        def visible
          attributes[:visible]
        end

        def visible=(visible)
          attributes[:visible] = visible
        end

        private

        # @private
        def rows
          attributes[:rows] ||= []
        end

      end

    end
  end
end